@@ -107,7 +107,7 @@ function(x, w=3, fun="sum", ..., na.policy="all", fillvalue=NA, expand=FALSE, si
 		if (length(nms) == nlyr(out)) {
 			names(out) <- nms
 		}
-		b <- writeStart(out, filename, overwrite, n=msz*4, sources=sources(x), wopt=wopt)
+		b <- writeStart(out, filename, overwrite, n=msz+2, sources=sources(x), wopt=wopt)
 		opt <- spatOptions()
 
 		for (i in 1:b$n) {
@@ -254,7 +254,7 @@ function(x, w=3, fun=mean, ..., na.policy="all", fillvalue=NA, pad=FALSE, padval
 			names(out) <- rep_len(nms, outnl) 
 		}
 	}
-	b <- writeStart(out, filename, overwrite, n=msz*4, sources=sources(x), wopt=wopt)
+	b <- writeStart(out, filename, overwrite, n=msz+2, sources=sources(x), wopt=wopt)
 
 	nread <- prod(w[1:2])
 
@@ -274,7 +274,13 @@ function(x, w=3, fun=mean, ..., na.policy="all", fillvalue=NA, pad=FALSE, padval
 			}
 		} else {
 			v <- lapply(1:w[3],
-				function(k) matrix(x[[k]]@cpp$focalValues(w, fillvalue, b$row[i]-1, b$nrows[i], opt), ncol=nc))
+				function(k) {
+					y <- x[[k]]
+					z <- y@cpp$focalValues(w, fillvalue, b$row[i]-1, b$nrows[i], opt)
+					y <- messages(y, "focal3D")
+					matrix(z, ncol=nc)
+				})
+				
 			v <- do.call(rbind, v)
 		}
 		for (j in startlyr:endlyr) {
@@ -372,7 +378,7 @@ function(x, w=3, fun, ..., fillvalue=NA, silent=TRUE, filename="", overwrite=FAL
 		wopt$names <- colnames(test)
 	}
 	out <- rast(x, nlyr=outnl)
-	b <- writeStart(out, filename, overwrite, n=msz*4, sources=sources(x), wopt=wopt)
+	b <- writeStart(out, filename, overwrite, n=msz+2, sources=sources(x), wopt=wopt)
 
 	nc <- ncol(out)
 	for (i in 1:b$n) {
@@ -576,7 +582,7 @@ function(x, w=3, fun="ols", ..., fillvalue=NA, filename="", overwrite=FALSE, wop
 	}
 	out <- rast(x, nlyr=outnl)
 	
-	b <- writeStart(out, filename, n=msz*4, sources=sources(x), wopt=wopt)
+	b <- writeStart(out, filename, n=msz+2, sources=sources(x), wopt=wopt)
 	ry <- x[[1]]
 	rx <- x[[-1]]
 
@@ -728,7 +734,7 @@ function(x, w=3, fun, ..., fillvalue=NA, filename="", overwrite=FALSE, wopt=list
 	outnl <- (nlyr(x) - 1) * length(test)
 	out <- rast(x, nlyr=outnl)
 
-	b <- writeStart(out, filename, n=msz*4, sources=sources(x), wopt=wopt)
+	b <- writeStart(out, filename, n=msz+2, sources=sources(x), wopt=wopt)
 
 	for (i in 1:b$n) {
 		v <- list()
