@@ -1,4 +1,46 @@
 
+buffer2 <- function(x, width, quadsegs=10) {
+	if (is.character(width)) {
+		if (!(width %in% names(x))) {
+			error("buffer2", paste(width, "is not a field in x"))
+		}
+		width <- x[[width, drop=TRUE]]
+	}
+	if (!is.numeric(width)) {
+		error("buffer2", "width is not numeric")
+	}
+	x@cpp <- x@cpp$buffer2(width, quadsegs)
+	messages(x, "buffer2")
+}
+
+
+buffer3 <- function(x, width, quadsegs=10) {
+	if (is.character(width)) {
+		if (!(width %in% names(x))) {
+			error("buffer3", paste(width, "is not a field in x"))
+		}
+		width <- x[[width, drop=TRUE]]
+	}
+	if (!is.numeric(width)) {
+		error("buffer3", "width is not numeric")
+	}
+	x@cpp <- x@cpp$buffer3(width, quadsegs)
+	messages(x, "buffer3")
+}
+
+buffer4 <- function(x, width, quadsegs=10) {
+	if (is.character(width)) {
+		if (!(width %in% names(x))) {
+			error("buffer4", paste(width, "is not a field in x"))
+		}
+		width <- x[[width, drop=TRUE]]
+	}
+	if (!is.numeric(width)) {
+		error("buffer4", "width is not numeric")
+	}
+	x@cpp <- x@cpp$buffer4(width, quadsegs)
+	messages(x, "buffer4")
+}
 
 roundtrip <- function(x, coll=FALSE) {
 	if (coll) {
@@ -169,8 +211,12 @@ setMethod("union", signature(x="SpatVector", y="missing"),
 
 setMethod("union", signature(x="SpatVector", y="SpatVector"),
 	function(x, y) {
-		x@cpp <- x@cpp$union(y@cpp)
-		messages(x, "union")
+		if (geomtype(x) != "polygons") {
+			unique(rbind(x, y))
+		} else {
+			x@cpp <- x@cpp$union(y@cpp)
+			messages(x, "union")
+		}
 	}
 )
 
@@ -261,6 +307,7 @@ setMethod("buffer", signature(x="SpatVector"),
 )
 
 
+
 setMethod("crop", signature(x="SpatVector", y="ANY"),
 	function(x, y, ext=FALSE) {
 		if (ext) {
