@@ -3,10 +3,10 @@
 
 .create_options <- function() {
 	opt <- methods::new("SpatOptions")
-	opt@pnt <- SpatOptions$new()
+	opt@cpp <- SpatOptions$new()
 	# check=T does not exist in ancient R
 	tmpdir <- try(tempdir(check = TRUE), silent=TRUE)
-	opt@pnt$tempdir <- normalizePath(tempdir(), winslash="/")
+	opt@cpp$tempdir <- normalizePath(tempdir(), winslash="/")
 	.terra_environment$options <- opt
 	.terra_environment$devs <- NULL
 
@@ -18,7 +18,7 @@
 }
 
 .option_names <- function() {
-	c("progress", "progressbar", "tempdir", "memfrac", "memmax", "memmin", "datatype", "filetype", "filenames", "overwrite", "todisk", "names", "verbose", "NAflag", "statistics", "steps", "ncopies", "tolerance", "pid", "threads", "scale", "offset") #, "append")
+	c("progress", "progressbar", "tempdir", "memfrac", "memmax", "memmin", "datatype", "filetype", "filenames", "overwrite", "todisk", "names", "verbose", "NAflag", "statistics", "steps", "ncopies", "tolerance", "tmpfile", "threads", "scale", "offset") #, "append")
 }
 
 
@@ -78,7 +78,7 @@
 defaultOptions <- function() {
 	## work around onLoad problem
 	if (is.null(.terra_environment$options)) .create_options()
-	.terra_environment$options@pnt$deepcopy()
+	.terra_environment$options@cpp$deepcopy()
 }
 
 spatOptions <- function(filename="", overwrite=FALSE, ..., wopt=NULL) {
@@ -88,8 +88,8 @@ spatOptions <- function(filename="", overwrite=FALSE, ..., wopt=NULL) {
 	## work around onLoad problem
 	if (is.null(.terra_environment$options)) .create_options()
 
-	opt <- .terra_environment$options@pnt$deepcopy()
-	opt$pid <- Sys.getpid()
+	opt <- .terra_environment$options@cpp$deepcopy()
+	opt$tmpfile <- paste0(gsub("^file", "", basename(tempfile())),  "_", Sys.getpid())
 	filename <- .fullFilename(filename, mustExist=TRUE)
 	if (!is.null(unlist(wopt))) {
 		wopt$filenames <- filename
@@ -126,7 +126,7 @@ spatOptions <- function(filename="", overwrite=FALSE, ..., wopt=NULL) {
 	opt <- spatOptions()
 	nms <- names(opt)
 	nms <- nms[!grepl("^\\.", nms)]
-	nms <- nms[!(nms %in% c("initialize", "messages", "getClass", "finalize", "datatype_set", "pid", "statistics", "gdal_options", "scale", "offset", "threads", "filenames", "NAflag"))]
+	nms <- nms[!(nms %in% c("initialize", "messages", "getClass", "finalize", "datatype_set", "tmpfile", "statistics", "gdal_options", "scale", "offset", "threads", "filenames", "NAflag"))]
 	defnms <- grepl("^def_", nms)
 	nms <- nms[!defnms]
 	out <- sapply(nms, function(n) eval(parse(text=paste0("opt$", n))))
@@ -159,7 +159,7 @@ spatOptions <- function(filename="", overwrite=FALSE, ..., wopt=NULL) {
 terraOptions <- function(..., print=TRUE) {
 	dots <- list(...)
 	if (is.null(.terra_environment$options)) .create_options()
-	opt <- .terra_environment$options@pnt
+	opt <- .terra_environment$options@cpp
 
 	nms <- names(dots)
 
@@ -202,6 +202,6 @@ terraOptions <- function(..., print=TRUE) {
 			warn("terraOptions", "memfrac > 0.9")
 		}
 	}
-	.terra_environment$options@pnt <- opt
+	.terra_environment$options@cpp <- opt
 }
 
