@@ -1,4 +1,3 @@
-
 # these methods require the dev version of leaflet
 
 checkLeafLetVersion <- function() {
@@ -104,11 +103,11 @@ baselayers <- function(tiles, wrap=TRUE) {
 setMethod("plet", signature(x="SpatVector"),
 	function(x, y="", col, fill=0.2, main=y, cex=1, lwd=2, border="black", alpha=1, popup=TRUE, label=FALSE, split=FALSE, tiles=c("Streets", "Esri.WorldImagery", "OpenTopoMap"), wrap=TRUE, legend="bottomright", collapse=FALSE, type=NULL, breaks=NULL, breakby="eqint", sort=TRUE, decreasing=FALSE, map=NULL, ...)  {
 
-		checkLeafLetVersion()
+		#checkLeafLetVersion()
 		y <- unique(y)
 		if (length(y) > 1) {
 			y = y[1]
-#			xvc <- svc(lapply(y, \(i) x[,i]))
+#			xvc <- svc(lapply(y, function(i) x[,i]))
 #			if (is.numeric(y)) {
 #				names(xvc) <- names(x)[y]
 #			} else {
@@ -133,7 +132,7 @@ setMethod("plet", signature(x="SpatVector"),
 		g <- geomtype(x)
 		leg <- NULL
 		if (y == "") { # no legend
-			group <- x@pnt$layer
+			group <- x@cpp$layer
 			if (group == "") group = g
 			cols <- .getCols(nrow(x), col)
 			pop  <- lab <- NULL
@@ -241,7 +240,7 @@ setMethod("plet", signature(x="SpatVector"),
 setMethod("plet", signature(x="SpatVectorCollection"),
 	function(x, col, fill=0, cex=1, lwd=2, border="black", alpha=1, popup=TRUE, label=FALSE, tiles=c("Streets", "Esri.WorldImagery", "OpenTopoMap"), wrap=TRUE, legend="bottomright", collapse=FALSE, map=NULL)  {
 
-		checkLeafLetVersion()
+		#checkLeafLetVersion()
 
 		if (is.null(map)) {
 			tiles <- unique(as.character(tiles))
@@ -364,6 +363,8 @@ setMethod("points", signature(x="leaflet"),
 
 
 make.panel <- function(x, maxcell) {
+
+	nl <- nlyr(x)
 	x <- spatSample(x, maxcell/nl, "regular", as.raster=TRUE, warn=FALSE)
 	if (is.lonlat(x)) {
 		asp <- 1/cos((mean(ext(x)[3:4]) * pi)/180)
@@ -380,7 +381,6 @@ make.panel <- function(x, maxcell) {
 	#	}
 	#}
 
-	nl <- nlyr(x)
 	asp <- asp * nrow(x) / ncol(x)
 	nc <- ceiling(2*sqrt(nl/2) * asp)
 	nr <- ceiling(nl / nc)
@@ -415,7 +415,7 @@ make.panel <- function(x, maxcell) {
 setMethod("plet", signature(x="SpatRaster"),
 	function(x, y=1, col, alpha=0.8, main=names(x), tiles=NULL, wrap=TRUE, maxcell=500000, legend="bottomright", shared=FALSE, panel=FALSE, collapse=TRUE, map=NULL)  {
 
-		checkLeafLetVersion()
+		#checkLeafLetVersion()
 
 		if (is.na(crs(x))) {
 			error("plet", "x must have a crs")
