@@ -1,15 +1,15 @@
 
 
-setMethod("window<-", signature(x="SpatRaster"), 
+setMethod("window<-", signature(x="SpatRaster"),
 	function(x, value)  {
 		if (inherits(value, "SpatExtent")) {
 			value <- value * ext(x)
-			if (!(x@ptr$setWindow(value@ptr))) {
+			if (!(x@cpp$setWindow(value@cpp))) {
 				error("window<-,SpatRaster", "could not set window")
 			}
 			#warn("window<-", "using a window is experimental")
 		} else if (is.null(value) || is.na(value)) {
-			x@ptr$removeWindow()
+			x@cpp$removeWindow()
 		} else {
 			error("window<-", "'value' should be a SpatExtent, NULL or NA")
 		}
@@ -18,9 +18,9 @@ setMethod("window<-", signature(x="SpatRaster"),
 )
 
 
-setMethod("window", signature(x="SpatRaster"), 
+setMethod("window", signature(x="SpatRaster"),
 	function(x)  {
-		x@ptr$hasWindow()
+		x@cpp$hasWindow()
 	}
 )
 
