@@ -32,6 +32,67 @@
 #include "recycle.h"
 #include <sstream>
 
+
+SpatGeom getPolygonsGeom2(OGRGeometry *poGeometry) {
+	SpatGeom g(polygons);
+	OGRPoint ogrPt;
+//	OGRwkbGeometryType geomtype = poGeometry->getGeometryType();
+//	if ( geomtype == wkbPolygon ) {
+		OGRPolygon *poGeom = ( OGRPolygon * )poGeometry;
+		OGRLinearRing *poRing = poGeom->getExteriorRing();
+		unsigned np = poRing->getNumPoints();
+		std::vector<double> X(np);
+		std::vector<double> Y(np);
+		for (size_t i=0; i<np; i++) {
+			poRing->getPoint(i, &ogrPt);
+			X[i] = ogrPt.getX();
+			Y[i] = ogrPt.getY();
+		}
+		SpatPart p(X, Y);
+		unsigned nh = poGeom->getNumInteriorRings();
+		for (size_t i=0; i<nh; i++) {
+			OGRLinearRing *poHole = poGeom->getInteriorRing(i);
+			unsigned np = poHole->getNumPoints();
+			std::vector<double> X(np);
+			std::vector<double> Y(np);
+			for (size_t j=0; j<np; j++) {
+				poHole->getPoint(j, &ogrPt);
+				X[j] = ogrPt.getX();
+				Y[j] = ogrPt.getY();
+			}
+			p.addHole(X, Y);
+		}
+		g.addPart(p);
+//	}
+	return g;
+}
+
+
+SpatVector SpatVector::buffer3(std::vector<double> d, unsigned quadsegs) {
+
+	SpatVector out;
+	recycle(d, size());
+	GDALDataset* v = write_ogr("", "layer", "Memory", false, true, std::vector<std::string>());
+
+	OGRLayer *poLayer = v->GetLayer(0);
+	poLayer->ResetReading();
+
+	OGRFeature *poFeature;
+	while( (poFeature = poLayer->GetNextFeature()) != NULL ) {
+		OGRGeometry *poGeometry = poFeature->GetGeometryRef();
+		if (poGeometry != NULL) {
+			// using d[0] for now
+			OGRGeometry *bufGeom = poGeometry->Buffer(d[0], quadsegs);
+			SpatGeom g = getPolygonsGeom2(bufGeom);
+			out.addGeom(g);
+		}
+		OGRFeature::DestroyFeature( poFeature );
+	}
+	GDALClose(v);
+	return out;
+}
+
+
 SpatVector SpatRaster::dense_extent(bool inside, bool geobounds) {
 
 	SpatExtent e = getExtent();
@@ -435,6 +496,10 @@ SpatRaster SpatRaster::warper(SpatRaster x, std::string crs, std::string method,
 		out.source[0].timezone = getTimeZone();
 		out.source[0].time = getTime();
 	}
+	if (hasUnit()) {
+		out.source[0].hasUnit = true;
+		out.source[0].unit = getUnit();
+	}
 
 	bool use_crs = !crs.empty();
 	if (use_crs) {
@@ -535,7 +600,6 @@ SpatRaster SpatRaster::warper(SpatRaster x, std::string crs, std::string method,
 
 	double halfy = out.yres() / 2;
 	for (size_t i = 0; i < out.bs.n; i++) {
-		int bandstart = 0;
 		eout.ymax = out.yFromRow(out.bs.row[i]) + halfy;
 		eout.ymin = out.yFromRow(out.bs.row[i] + out.bs.nrows[i]-1) - halfy;
 		SpatRaster crop_out = out.crop(eout, "near", false, sopt);
@@ -545,6 +609,7 @@ SpatRaster SpatRaster::warper(SpatRaster x, std::string crs, std::string method,
 			return crop_out;
 		}
 
+		int bandstart = 0;
 		for (size_t j=0; j<ns; j++) {
 			GDALDatasetH hSrcDS;
 			if (!open_gdal(hSrcDS, j, false, sopt)) {
@@ -589,8 +654,10 @@ SpatRaster SpatRaster::warper(SpatRaster x, std::string crs, std::string method,
 			out.setError("cannot do this transformation (warp)");
 			return out;
 		}
-		std::vector<double> v = crop_out.getValues(-1, opt);
-		if (!out.writeBlock(v, i)) return out;
+//		std::vector<double> v = crop_out.getValues(-1, opt);
+//		if (!out.writeBlock(v, i)) return out;
+		if (!out.writeBlock(crop_out.source[0].values, i)) return out;
+
 	}
 	out.writeStop();
 	if (mask) {
@@ -980,24 +1047,8 @@ SpatRaster SpatRaster::rectify(std::string method, SpatRaster aoi, unsigned usea
 	}
 	double gt[6];
 	if( poDataset->GetGeoTransform(gt) != CE_None ) {
-	
 		if (GCP_geotrans(poDataset, gt)) {
-		
-			GDALClose( (GDALDatasetH) poDataset );
-			std::string tmpfile = tempFile(opt.get_tempdir(), opt.pid, "_rect.tif");
-			//++17 
-			//std::filesystem::copy_file(source[0].filename, tmpfile);	
-
-			std::ifstream  src(source[0].filename, std::ios::binary);
-			std::ofstream  dst(tmpfile,   std::ios::binary);
-			dst << src.rdbuf();	
-		
-			GDALDataset *poDataset = openGDAL(tmpfile, GDAL_OF_RASTER | GDAL_OF_READONLY, source[0].open_drivers, source[0].open_ops);
 			poDataset->SetGeoTransform(gt);
-			GDALClose( (GDALDatasetH) poDataset );
-			SpatRaster tmp(tmpfile,  {-1}, {""}, {}, {});
-			return tmp.rectify(method, aoi, useaoi, snap, opt);
-
 		} else {
 			out.setError("can't get the geotransform");
 			GDALClose( (GDALDatasetH) poDataset );
@@ -1218,7 +1269,7 @@ SpatRaster SpatRaster::rgb2col(size_t r,  size_t g, size_t b, SpatOptions &opt)
 		if (canProcessInMemory(opt)) {
 			driver = "MEM";
 		} else {
-			filename = tempFile(opt.get_tempdir(), opt.pid, ".tif");
+			filename = tempFile(opt.get_tempdir(), opt.tmpfile, ".tif");
 			opt.set_filenames({filename});
 			driver = "GTiff";
 		}
@@ -1390,7 +1441,7 @@ SpatRaster SpatRaster::viewshed(const std::vector<double> obs, const std::vector
 		}
 	}
 
-	std::string filename = tempFile(topt.get_tempdir(), topt.pid, ".tif");
+	std::string filename = tempFile(topt.get_tempdir(), topt.tmpfile, ".tif");
 	driver = "GTiff";
 
 	GDALDatasetH hSrcDS;
@@ -1471,7 +1522,7 @@ SpatRaster SpatRaster::proximity(double target, double exclude, bool keepNA, std
 		if (canProcessInMemory(opt)) {
 			driver = "MEM";
 		} else {
-			filename = tempFile(opt.get_tempdir(), opt.pid, ".tif");
+			filename = tempFile(opt.get_tempdir(), opt.tmpfile, ".tif");
 			opt.set_filenames({filename});
 			driver = "GTiff";
 		}
@@ -1549,7 +1600,7 @@ SpatRaster SpatRaster::proximity(double target, double exclude, bool keepNA, std
 		return out;
 	}
 
-	std::string tmpfile = tempFile(opt.get_tempdir(), opt.pid, ".tif");
+	std::string tmpfile = tempFile(opt.get_tempdir(), opt.tmpfile, ".tif");
 	std::string fname = mask ? tmpfile : filename;
 	if (!out.create_gdalDS(hDstDS, fname, driver, false, 0, {false}, {1}, {0}, ops)) {
 		out.setError("cannot create new dataset");
@@ -1617,7 +1668,7 @@ SpatRaster SpatRaster::sieveFilter(int threshold, int connections, SpatOptions &
 		if (canProcessInMemory(opt)) {
 			driver = "MEM";
 		} else {
-			filename = tempFile(opt.get_tempdir(), opt.pid, ".tif");
+			filename = tempFile(opt.get_tempdir(), opt.tmpfile, ".tif");
 			opt.set_filenames({filename});
 			driver = "GTiff";
 		}
