@@ -154,6 +154,7 @@ class SpatVector {
 				addWarning("Cannot set SRS to vector: "+ msg);
 				return false;
 			}
+			source = "";
 			return true;	
 		}
 
@@ -213,7 +214,7 @@ class SpatVector {
 		SpatVector set_holes(SpatVector x, size_t i);
 		SpatVector remove_duplicate_nodes(int digits);
 		
-		bool read(std::string fname, std::string layer, std::string query, std::vector<double> extent, SpatVector filter, bool as_proxy, std::string what);
+		bool read(std::string fname, std::string layer, std::string query, std::vector<double> extent, SpatVector filter, bool as_proxy, std::string what, std::vector<std::string> options);
 		
 		bool write(std::string filename, std::string lyrname, std::string driver, bool append, bool overwrite, std::vector<std::string>);
 
@@ -330,9 +331,16 @@ class SpatVector {
 		SpatVector aggregate(bool dissolve);
 		SpatVector aggregate(std::string field, bool dissolve);
 
+		SpatVector buffer2(std::vector<double> d, unsigned quadsegs);
+		SpatVector buffer3(std::vector<double> d, unsigned quadsegs);
+//		SpatVector buffer4(double d);
+//		SpatVector bufferclip(std::vector<double> d, std::string jointype, double miter_limit, int precision, double arc_tolerance);
+		
 		SpatVector buffer(std::vector<double> d, unsigned quadsegs, std::string capstyle, std::string joinstyle, double mitrelimit, bool singlesided);
+		SpatVector buffer_lonlat(std::string vt, std::vector<double> d, unsigned quadsegs);
 
-		SpatVector point_buffer(std::vector<double>	 d, unsigned quadsegs, bool no_multipolygons);
+
+		SpatVector point_buffer(std::vector<double>	d, unsigned quadsegs, bool no_multipolygons, bool wrap);
 
 		SpatVector centroid(bool check_lonlat);
 		SpatVector point_on_surface(bool check_lonlat);
